<?php namespace awp;

class AdminPages extends Module
{

    public static function DisplaySettingsSection($page)
    {
        global $wp_settings_sections, $wp_settings_fields;
        if ( ! isset( $wp_settings_sections[ $page ] ) ) {
            return;
        }

        foreach ( (array) $wp_settings_sections[ $page ] as $section ) 
        {
            if ( $section['title'] ) {
                echo "<h2>{$section['title']}</h2>\n";
            }
    
            if ( $section['callback'] ) {
                call_user_func( $section['callback'], $section );
            }
    
            if ( ! isset( $wp_settings_fields ) || ! isset( $wp_settings_fields[ $page ] ) || ! isset( $wp_settings_fields[ $page ][ $section['id'] ] ) ) {
                continue;
            }

            //do_settings_fields( $page, $section['id'] );
            $sectionId = $section['id'];

            /* do_settings_fields_properly*/
            if ( ! isset( $wp_settings_fields[ $page ][ $sectionId ] ) ) {
                return;
            }
        
            foreach ( (array) $wp_settings_fields[ $page ][ $sectionId ] as $field ) {
                call_user_func( $field['callback'], $field['args'] );
            }
            /* do_settings_fields_properly END*/
        }

    }

    public static function IncludeFormHeader($option_group, $action="update")
    {
        echo "<input type='hidden' name='option_page' value='" . esc_attr( $option_group ) . "' />";
        echo "<input type=\"hidden\" name=\"action\" value=\"{$action}\" />";
        wp_nonce_field( "$option_group-options" );
    }

    public static function CreateFields($page, $fields, $settingName, $sectionName)
    {
        
        $fieldKeys = array_keys($fields);
    
        for($i=0;$i<count($fieldKeys);$i++)
        {
            $key = $fieldKeys[$i];
            $item = (object)$fields[$key];

            $item->key = $key;
            $item->setting = $settingName;
            $item->section = $sectionName;
            
            switch($item->type)
            {
                case "hidden":
                    add_settings_field($key, $item->label, function($output) use($item){\awp\App::Instance()->DrawHiddenInput($item, $output);}, $page, $sectionName );
                    break;
                case "bool":
                case "boolean":
                    add_settings_field($key, $item->label, function($output) use($item){\awp\App::Instance()->DrawTogleSwitch($item, $output);}, $page, $sectionName );
                    break;
                case "array":
                    add_settings_field($key, $item->label, function($output) use($item){\awp\App::Instance()->DrawListArea($item, $output);}, $page, $sectionName );
                    break;
                case "text":
                case "string":
                    add_settings_field($key, $item->label, function($output) use($item){\awp\App::Instance()->DrawTextInput($item, $output);}, $page, $sectionName );
                    break;
                case "textarea":
                    add_settings_field($key, $item->label, function($output) use($item){\awp\App::Instance()->DrawTextArea($item, $output);}, $page, $sectionName );
                    break;
                default:
                    add_settings_field($key, $item->label, function($output) use($item){print_r($item);}, $page, $sectionName );
            }
            
        }
    }




}

?>