<?php  namespace awp;
if(class_exists(__NAMESPACE__."\Cleanup"))
{
    return;
}


/** citations and stolen code
 * 
 * https://developer.wordpress.org/
 * Documentation for WP
 * 
 * https://blacksaildivision.com/how-to-clean-up-wordpress-head-tag
 * The RemoveDefaultHeadDAta() and DisableDefaultFeatures() (including functions called feom them) come from this website, i have refactored them to to be compact and OOP friendly.
 * 
 * https://kinsta.com/knowledgebase/disable-emojis-wordpress/#disable-emojis-code
 * DisableEmojis()
 * 
 * https://wordpress.stackexchange.com/questions/83887/return-current-page-type
 * PageType()
 * 
 * https://wordpress.stackexchange.com/questions/233140/how-can-i-get-a-list-of-all-enqueued-scripts-and-styles
 * EnquedItems()
 * 
 */

class Cleanup extends Module
{

	public function __construct()
	{ 
		add_action( "DisableTagOptions", array($this, "DisableTagOptions") );
        $this->EnableTagOptions();

		//cleanup
		add_action( "RemoveDefaultIncludes", array($this, "RemoveDefaultIncludes") );
		do_action("RemoveDefaultIncludes");
		add_action( "RemoveAdminBar", array($this, "RemoveAdminBar") );
		add_action( "RemoveDefaultHeadData", array($this, "RemoveDefaultHeadData") );
		do_action("RemoveDefaultHeadData");
		add_action( "DisableDefaultFeatures", array($this, "DisableDefaultFeatures") );
		do_action("DisableDefaultFeatures");

		//debug functions
		add_action( "WriteEnqued", array($this, "WriteEnqued") );
		add_action( "ConsolePageData", array($this, "ConsolePageData") );

	}
	
	public function EnableTagOptions()
	{
		add_filter( "style_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
		add_filter( "script_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
	}

	public function DisableTagOptions()
	{
		remove_filter( "style_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
		remove_filter( "script_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
	}
    
	public function GetDefaultIncludes()
	{
		return array("emoji", "comment-reply", "wp-block-library", "wp-block-library-theme", "global-styles");
	}
    
    public function GetDefaultHeadTags()
    {
        return array("wp-dns-prefetch", "oEmbeds", "rss-feed", "rest-links", "xml-rpc", "wlwmanifest_link", "wp_generator");
    }

    public function GetDefaultFeatures()
    {
        return array("rss-feed", "rest-api", "xml-rpc");
    }

    public function DisplayConfigPage()
    {
        require_once "info.php";
    }

	public function CustomTagOptions($tag, $handle, $src)
	{
		//Find all attributes marked with custom tags, if none continue as norma.
		$customTags = ["%5B", "%5D"]; // URL SAFE VERSION OF [ AND ]
		$attRegEx = "/{$customTags[0]}(.*)(?:(=)(.*))?{$customTags[1]}/U";
		$matches = [];
		if(!preg_match_all ($attRegEx, $src, $matches, PREG_UNMATCHED_AS_NULL|PREG_SET_ORDER ))
				return $tag;

		//Grab the tag name and the close tag+method*. *could be ></script> OR /> or >
		$tagEnds;
		$tagRegEx = "/^<(?<tag>\w+).*?(>[^>]*<\/\k'tag'>|\/>|>)$/";
		preg_match($tagRegEx ,$tag, $tagEnds);
		
		//stringyfy new attributes and replace custom tags with new string.
		$attributesString = "";
		$changeType = false;
		for($i=0; $i<count($matches); $i++)
		{
			$src = str_replace ($matches[$i][0], "", $src);
			$attributesString .= " {$matches[$i][1]}" . (isset($matches[$i][3]) ? "{$matches[$i][2]}\"" . trim($matches[$i][3], "'\"`‘’") . "\"" : "");
			if(strtolower($matches[$i][1]) == "type")
				$changeType = true;
		}

		//build new tag and add default js type to make code less complainy.
		return "<{$tagEnds[1]} " . (!$changeType && $tagEnds[1]=="script" ? "type=\"text/javascript\" " : "") . ($tagEnds[1]=="script" ? "src": "href") . "=\"" . esc_url($src) ."\"{$attributesString}{$tagEnds[2]}\n";
	}

	public function RemoveAdminBar()
	{
		
		show_admin_bar(false);
		
		add_action( 'wp_enqueue_scripts', function()
		{  
			//wp_dequeue_script("admin-bar");
			//wp_dequeue_style("admin-bar");
		}, 11);
	}


    public function DisableDefaultFeatures()
    {
        
		add_action( 'init', function()
		{  
            $defaultDefaultFeatures = apply_filters( "ModifyDefaultFeatures", $this->GetDefaultFeatures() );
			for($i=0; $i<count($defaultDefaultFeatures); $i++)
			{
				$defaultName = $defaultDefaultFeatures[$i];

				switch($defaultName)
				{
					case  "rss-feed":
                        $this->DisableRSS();
                        break;
                    case  "rest-api":
                        $this->DisableRest();
                        break;
                    case  "xml-rpc":
                        add_filter('xmlrpc_enabled', function () { return false; });
                        break;
				
				}
			}
		});
    }

	public function RemoveDefaultIncludes()
	{
		
		add_action( 'init', function()
		{  
			$defaultIncludes = apply_filters( "ModifyDefaultIncludes", $this->GetDefaultIncludes() );
			for($i=0; $i<count($defaultIncludes); $i++)
			{
				$defaultName = $defaultIncludes[$i];

				switch($defaultName)
				{
					case  "emoji":
						$this->DisableEmojis();
						break;
					case "admin-bar":
						$this->RemoveAdminBar();
						break;
					case "comment-reply":
						add_action( 'wp_enqueue_scripts', function() use ($defaultName){ wp_dequeue_script($defaultName); });
						break;
					case "wp-block-library":
                    case "wp-block-library-theme":
					case "global-styles":
						add_action( 'wp_enqueue_scripts', function() use ($defaultName){ wp_dequeue_style($defaultName); });
						break;
					default:
						add_action( 'wp_enqueue_scripts', function() use ($defaultName){ wp_dequeue_script($defaultName); });
						add_action( 'wp_enqueue_scripts', function() use ($defaultName){ wp_dequeue_style($defaultName); });
				
				}
			}
		});
	}

	public function RemoveDefaultHeadData()
	{
		
		add_action( 'init', function()
		{  
            $defaultDefaultHeadTags = apply_filters( "ModifyDefaultHeadTags", $this->GetDefaultHeadTags() );
			for($i=0; $i<count($defaultDefaultHeadTags); $i++)
			{
				$defaultName = $defaultDefaultHeadTags[$i];

				switch($defaultName)
				{
					case  "wp-dns-prefetch":
                        $this->DisableWPDNSPrefetch();
                        break;
                    case "oEmbeds":
                        $this->DisableOEmbeds();
                        break;
					case  "rss-feed":
                        remove_action( 'wp_head', 'feed_links_extra', 3 ); // Display the links to the extra feeds such as category feeds
                        remove_action( 'wp_head', 'feed_links', 2 ); // Display the links to the general feeds: Post and Comment Feed
                        break;
                    case "rest-links":
                        remove_action('wp_head', 'rest_output_link_wp_head');
					case  "xml-rpc":
                        remove_action( 'wp_head', 'rsd_link' ); // Display the link to the Really Simple Discovery service endpoint, EditURI link
                        break;
					case  "wlwmanifest_link":
		                remove_action('wp_head', 'wlwmanifest_link');
                        break;
					case  "wp_generator":
                        remove_action('wp_head', 'wp_generator');
                        break;
                    case "blog-links":
                        remove_action( 'wp_head', 'index_rel_link' ); // index link
                        remove_action( 'wp_head', 'parent_post_rel_link', 10, 0 ); // prev link
                        remove_action( 'wp_head', 'start_post_rel_link', 10, 0 ); // start link
                        remove_action( 'wp_head', 'adjacent_posts_rel_link', 10, 0 ); // Display relational links for the posts adjacent to the current post.
                        break;
				
				}
			}
        
		});

	}


    private function DisableRSS()
    {
        foreach (['do_feed_rss2', 'do_feed_rss2_comments'] as $feedAction) {
            add_action($feedAction, function (): void {
                wp_redirect(home_url(), 301);
                exit;
            }, 1);
        }
    }

    private function DisableRest()
    {
		//Disable REST-API for all users except of admin
		add_filter('rest_authentication_errors', function ($access) {
			if (!current_user_can('administrator')) {
				return new WP_Error('rest_cannot_access', 'Only authenticated users can access the REST API.', ['status' => rest_authorization_required_code()]);
			}
			return $access;
		});
    }

	private function DisableOEmbeds() {

		// Remove the REST API endpoint.
		remove_action( 'rest_api_init', 'wp_oembed_register_route' );
	   
		// Turn off oEmbed auto discovery.
		add_filter( 'embed_oembed_discover', '__return_false' );
	   
		// Don't filter oEmbed results.
		remove_filter( 'oembed_dataparse', 'wp_filter_oembed_result', 10 );
	   
		// Remove oEmbed discovery links.
		remove_action( 'wp_head', 'wp_oembed_add_discovery_links' );
	   
		// Remove oEmbed-specific JavaScript from the front-end and back-end.
		remove_action( 'wp_head', 'wp_oembed_add_host_js' );
		add_filter( 'tiny_mce_plugins', function ($plugins) {
			return array_diff($plugins, array('wpembed'));
		} );
	   
		// Remove all embeds rewrite rules.
		add_filter( 'rewrite_rules_array', function ($rules) {
			foreach($rules as $rule => $rewrite) {
				if(false !== strpos($rewrite, 'embed=true')) {
					unset($rules[$rule]);
				}
			}
			return $rules;
		} );
	   
		// Remove filter of the oEmbed result before any HTTP requests are made.
		remove_filter( 'pre_oembed_result', 'wp_filter_pre_oembed_result', 10 );
	}
    
	private function DisableEmojis() {
		remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
		remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
		remove_action( 'wp_print_styles', 'print_emoji_styles' );
		remove_action( 'admin_print_styles', 'print_emoji_styles' ); 
		remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
		remove_filter( 'comment_text_rss', 'wp_staticize_emoji' ); 
		remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
		//DisableEmojisTinyMCE - compressed to avoid exposing public funtions
		add_filter( 'tiny_mce_plugins',  function($plugins){
			if ( is_array( $plugins ) )
				return array_diff( $plugins, array( 'wpemoji' ) );
			return array();
		});
		//DisableEmojisRemoveDNSPrefetch - compressed to avoid exposing public funtions
		add_filter( 'wp_resource_hints', function($urls, $relation_type){
			if ( 'dns-prefetch' == $relation_type ) {
				$emoji_svg_url = apply_filters( 'emoji_svg_url', 'https://s.w.org/images/core/emoji/2/svg/' );
				$urls = array_diff( $urls, array( $emoji_svg_url ) );
			}
			return $urls;
		}, 10, 2 );
	}
	
    private function DisableWPDNSPrefetch()
    {
		// Remove s.w.org entry
		add_filter('wp_resource_hints', function (array $urls, string $relation): array {
			if ($relation == 'dns-prefetch') {
				$urls = array_filter($urls, function (string $url): bool { return strpos($url, 's.w.org') === false; });
			}
			return $urls;
		}, 10, 2);
    }
	   

	public function WriteEnqued() 
	{
		add_action( 'wp_footer', function(){  print_r($this->EnquedItems()); });

	}

	public function ConsolePageData()
	{
		add_action( 'wp_enqueue_scripts', function()
		{

			$debugHandle = "debug_page_data";
			global $template, $wp_query;
			$t = $template;
	
			$content = [
				"pageName" => isset($wp_query->query_vars["name"]) ? $wp_query->query_vars["name"] : "",
				"pageType" => $this->PageType(),
				"template" => @end(explode("wp-content/", $template)),
				"styles" => $this->EnquedItems()['styles'],
				"scripts" => $this->EnquedItems()['scripts'],
				"postType" => get_post_type(),
				"postName" => get_post_field( 'post_name', get_post()),
			];
	
			wp_register_script( $debugHandle, '');
			wp_enqueue_script($debugHandle);
			wp_localize_script( $debugHandle, $debugHandle,  ($content));
			wp_add_inline_script($debugHandle, "console.log({$debugHandle});");

		} , 20 );
	}

	private function EnquedItems()
	{
		global $post;
		$result = [];
		$result['scripts'] = [];
		$result['styles'] = [];

		// Print all loaded Scripts
		global $wp_scripts;
		foreach( $wp_scripts->queue as $script ) :
		$result['scripts'][] =  $wp_scripts->registered[$script]->handle . ";";
		endforeach;

		// Print all loaded Styles (CSS)
		global $wp_styles;
		foreach( $wp_styles->queue as $style ) :
		$result['styles'][] =  $wp_styles->registered[$style]->handle . ";";
		endforeach;

		return $result;
	}

	private function PageType() {
		global $wp_query;
		$loop = 'notfound';

		if ( $wp_query->is_page ) {
			$loop = is_front_page() ? 'front' : 'page';
		} elseif ( $wp_query->is_home ) {
			$loop = 'home';
		} elseif ( $wp_query->is_single ) {
			$loop = ( $wp_query->is_attachment ) ? 'attachment' : 'single';
		} elseif ( $wp_query->is_category ) {
			$loop = 'category';
		} elseif ( $wp_query->is_tag ) {
			$loop = 'tag';
		} elseif ( $wp_query->is_tax ) {
			$loop = 'tax';
		} elseif ( $wp_query->is_archive ) {
			if ( $wp_query->is_day ) {
				$loop = 'day';
			} elseif ( $wp_query->is_month ) {
				$loop = 'month';
			} elseif ( $wp_query->is_year ) {
				$loop = 'year';
			} elseif ( $wp_query->is_author ) {
				$loop = 'author';
			} else {
				$loop = 'archive';
			}
		} elseif ( $wp_query->is_search ) {
			$loop = 'search';
		} elseif ( $wp_query->is_404 ) {
			$loop = 'notfound 404';
		}

		return $loop;
	}


    public function EnableTemplatePages()
    {
        add_filter('template_include', array($this, "AllowTemplatePages"));
    }

    public function AllowTemplatePages()
    {
        $config = (object)array("prefix"=>"page-", "suphix"=>".php", "urlInputs"=>0, "requestReg"=>"/^[\w\d\-_.?\/# ]+$/");
        $config = apply_filters("EditTemplatePageConfig", $config);

        $allowData = 0;
        $definedTemplates = array("default" => (object)array("urlInputs" => 0, "fileRename" => false));
        $definedTemplates = apply_filters("EditDefienedTempaltes", $definedPages);

        global $wp;
        $safeRequest;
        $file = @end(explode("/",$template));
        
        $request = stripslashes(urldecode($wp->request));
    
        $logicCheck = substr_count($request,"/") <= (int)$allowData;
    
        //var_dump($request );
        if($file == "404.php" && $logicCheck && preg_match("/^[\w\d\-_.?\/# ]+$/", $request, $safeRequest, PREG_UNMATCHED_AS_NULL))
        {
            
            $safeRequest = explode("/",$safeRequest[0]);
            $data = array();
            while(next($safeRequest) !== false && $logicCheck)
            {
                $data[] = current($safeRequest);
            }
            
            reset( $safeRequest );
            $safeRequest = $safeRequest[0];
            
            if($logicCheck)
                set_query_var("request_data", $data);
            
            
            $file = get_stylesheet_directory() . "/page-" . $safeRequest . ".php";
            if(file_exists($file))
            {
                add_action("pre_get_posts", function(){
                    global $wp_query;
                
                    status_header( 200 );
                    $wp_query->is_page = true;
                    $wp_query->is_404=false;
                    
                    //var_dump(get_query_var("request_data"));
                
                });
                return $file;
            }
        }
    
        return $template;
    }



}

?>