<?php namespace awp;

class Info extends Module
{
	public function __construct()
	{
		$this->RequireMod("cleanup");
		$this->RequireMod("form");
		$this->RequireMod("admin-pages");
		$this->RequireMod("page-loading");

        add_action( "admin_init", array($this, "LoadProperties") );
        add_action( "admin_menu", array($this, "BuildLandingPage") );
		add_action("muplugins_loaded", array($this, "ApplySettings") );

	}
	
	public function ApplySettings(){

		$filters = [
			"disableIncludes"=>"ModifyDefaultIncludes",
			"disableHeadData"=>"ModifyDefaultHeadTags",
			"disableFeatures"=>"ModifyDefaultFeatures",
		];

		$filterKeys = array_keys($filters);

		for($i=0; $i< count($filterKeys); $i++)
		{
			$key = $filterKeys[$i];
			$filter = $filters[$key];
			
			add_filter($filter, function($includes) use($key){
				$setting = get_option("ash-wp-settings", array());
				$setting = empty($setting) ? array() : $setting;
				$value = array_key_exists($key, $setting) ? $setting[$key] : null ;
				
				if($value == null)
					return $includes;

				//i only want to reorder for the check and then return original order.
				$icopy = $includes;
				$vcopy = $value;

				//order the copies only.
				sort($icopy);
				sort($vcopy);
				
				if($icopy == $vcopy)
					return $includes;

				return $value;
			});
		}

		$setting = get_option("ash-wp-settings", array());
		$setting = empty($setting) ? array() : $setting;
		$tagOptions = array_key_exists("tagOptions", $setting) ? $setting["tagOptions"] : null ;
		$adminBar = array_key_exists("adminBar", $setting) ? $setting["adminBar"] : null ;
		$pageData = array_key_exists("pageData", $setting) ? $setting["pageData"] : null ;
		$blockAuthors = array_key_exists("blockAuthors", $setting) ? $setting["blockAuthors"] : null ;
		$renderPageTemplates = array_key_exists("renderPageTemplates", $setting) ? $setting["renderPageTemplates"] : null ;


		if($tagOptions===false)
			do_action( "DisableTagOptions");

		if($adminBar===false)
		{
			add_filter("ModifyDefaultIncludes", function($includes){
				$includes[] = "admin-bar"; 
				return $includes;
			});
		}
		
		if($pageData===true)
			do_action( "ConsolePageData");

            
		if($blockAuthors===false)
            do_action( "AllowAuthors");

        
		if($renderPageTemplates===true)
            do_action( "RenderPageTemplates");

		//do_action( "WriteEnqued" );

	}


	public function BuildLandingPage(){

		$settignsPageHook = add_options_page( "Ashley's WordPress Library", "Ashley's WP Library", 'manage_options', 'ash-wp', function (){
		?>
		<div class="wrap">
				<h1><?=esc_html_e( 'Ashleys Wordpress Library Config Page', 'ash-wp' ); ?></h1>
				<form method="POST" action="options.php">
					<?php settings_fields( "ash-wp" ); ?>
					<?php $this->app->DisplaySettingsSection( "ash-wp" ); ?>
					<?php submit_button(); ?>
				</form>
			<div>
			<?php
		} );
		
		add_action("admin_enqueue_scripts", function($hook) use($settignsPageHook){
			if($hook !== $settignsPageHook)
				return;
			wp_enqueue_style( "ash-wp-style-form", plugin_dir_url( __FILE__ ) . "styles/forms.css", array(), null);
		});

	} 

	public function LoadProperties()
	{

		$settingName= "ash-wp-settings";
		$sectionName = "ash-wp-config";
		
		$configFields = [
			"tagOptions" => ["label"=>"Tag Options", "default"=>true, "type"=>"boolean", 
				"description"=>"Adds the ability to customise the tag when you use you wp_include_style/script, you can add attributes use [] in the url e.g. \"myfile.js[async][type=module].\" "],
			"disableIncludes" => ["label"=>"Remove Included files", "default"=>$this->app->GetDefaultIncludes(), "type"=>"array", 
				"description"=>""],
			"disableHeadData" => ["label"=>"Cleanup Head Tag Clutter", "default"=>$this->app->GetDefaultHeadTags(), "type"=>"array", 
				"description"=>""],
			"disableFeatures" => ["label"=>"Disable Advanced BlogFeatures", "default"=>$this->app->GetDefaultFeatures(), "type"=>"array", 
				"description"=>""],
			"adminBar" => ["label"=>"Admin Bar", "default"=>true, "type"=>"boolean", 
				"description"=>"Allows you to hide the admin bar, mainly for debuggin UI and fixed elements that hide"],
			"pageData" => ["label"=>"Page Data", "default"=>false, "type"=>"boolean", 
				"description"=>"Dumps a bunch of debug information in an object and logs it to the terminal."],
            "blockAuthors" => ["label"=>"Block Authour Pages", "default"=>true, "type"=>"boolean", 
                "description"=>"Removes access to /"],
            "renderPageTemplates" => ["label"=>"Render Page Templates", "default"=>false, "type"=>"boolean", 
                "description"=>"allows you to create a page tempalte file without adding a page thorough wp. example-page"],
            
        ];



		register_setting( "ash-wp", $settingName, array("sanitize_callback"=>function($data)use($configFields){ return $this->ParseInput($data, $configFields);}) );

		add_settings_section( $sectionName, "Config Section", null, "ash-wp" );
		

		$configKeys = array_keys($configFields);

		for($i=0;$i<count($configKeys);$i++)
		{
			$key = $configKeys[$i];
			$item = (object)$configFields[$key];
			$item->key = $key;
			$item->setting =$settingName;
			$item->section =$sectionName;
			
			switch($item->type)
			{
				case "bool":
				case "boolean":
					add_settings_field($key, $item->label, function($output) use($item){$this->app->DrawTogleSwitch($item, $output);}, "ash-wp", "ash-wp-config"  );
					break;
				case "array":
					add_settings_field($key, $item->label, function($output) use($item){$this->app->DrawListArea($item, $output);}, "ash-wp", "ash-wp-config"  );
			}
		}

	} 

	


	private function ParseInput($inputData, $fieldData)
	{

		if($inputData == null)
			$inputData = array();

		$fieldKeys = array_keys($fieldData);
		$output = array();

		for($i=0; $i<count($fieldKeys); $i++)
		{
			$key = $fieldKeys[$i];
			$fieldInfo = (object)$fieldData[$key];

			$fieldInput = array_key_exists($key, $inputData) ? $inputData[$key]: null;

			if($fieldInput !== null)
			{	
				switch($fieldInfo->type)
				{
					case "bool":
					case "boolean":
						$fieldInput= json_decode($fieldInput); //prone to errors
						break;
					case "array":
						$fieldInput = $this->TextAreaToArray($fieldInput);
						break;
					
				}
			}
			else
			{
				$fieldInput = $fieldInfo->default;
			}

			$output[$key] = $fieldInput;

		}
		
		return $output;
	}





	private function TextAreaToArray($value)
	{
		$newValue = $value;
		$newValue = sanitize_textarea_field($newValue);
		$newValue = explode("\n", $newValue);
		$newValue = array_map('trim', $newValue);


		return $newValue;
	}

}


