<?php namespace awp;

class PageLoading extends Module{

    public function __construct(){
        
		add_action( "AllowAuthors", array($this, "AllowAuthors") );
		$this->BlockAuthors();

		add_action( "RenderPageTemplates", array($this, "RenderPageTemplates") );

    }
    

	public function AllowAuthors()
	{
		remove_filter( "template_redirect", array($this, "BlockAllAuthors"), 10, 3 );
	}

	public function BlockAuthors()
	{
		add_filter( "template_redirect", array($this, "BlockAllAuthors"), 10, 3 );
	}

    public function BlockAllAuthors()
    {
        global $wp_query;

        if ( is_author() )
        {
            $this->Load404();
        }
    }

    public function RenderPageTemplates()
    {
        add_filter("template_include", array($this, "UnblockPageTemplate"), 10, 3 );
    }

    public function RemovePageTemplates()
    {
        remove_filter("template_include", array($this, "UnblockPageTemplate"), 10, 3 );
    }

    public function UnblockPageTemplate($template)
    {

        if(!is_404())
            return $template;

        global $wp, $wp_query;
        $request = stripslashes(urldecode($wp->request));
        $safeRequest;
        if(!preg_match("/^[\w\d\-_.?\/# ]+$/", $request, $safeRequest, PREG_UNMATCHED_AS_NULL))
            return $template;

        $pageMaxRequestSections = apply_filters("PageMaxRequestSections", $safeRequest);
        $pageMaxRequestSections  = $pageMaxRequestSections == $safeRequest ? 0 : intval($pageMaxRequestSections);

	    $sectionCheck = substr_count($request,"/") <= (int)$pageMaxRequestSections;
        if(!$sectionCheck)
            return $template;

        if($pageMaxRequestSections > 0)
        {
            $data = array();
            while(next($safeRequest) !== false)
            {
                $data[] = current($safeRequest);
            }
            
            reset( $safeRequest );
            set_query_var("request_sections", $data);
            //get_query_var("request_sections");
        }
        
        $safeRequest = $safeRequest[0];
        $file = get_stylesheet_directory() . "/page-" . $safeRequest . ".php";
        if(file_exists($file))
        {
            status_header( 200 );
			$wp_query->is_page = true;
			$wp_query->is_404 = false;

            add_filter('wp_title', function($title) use($safeRequest){
                if(empty($title))
                    $title = ucwords($safeRequest, "-");

                return $title;
            });


            return $file;

        }




        return $template;
    }

    public function Load404()
    {
        global $wp_query;
        $wp_query->set_404();
        status_header( 404 );
        get_template_part( 404 ); 
        exit();
    }


}